bl_info = {
    "name" : "Animate Collection Visibility",
    "author" : "Luminous Labs, written by raq/mirkan1",
    "description" : "Animate Viewport/Render Visibility",
    "blender" : (3, 0, 0),
    "location" : "View3D",
    "category" : "Generic"
}

import bpy
from time import sleep
            
class Collection_Iterator(bpy.types.Operator):
    bl_idname = "outliner.hide_renders_hide_viewpoints_operator"
    bl_label = "Animate Collection Visibility"
    
    @classmethod
    def poll(cls, context):
        return context.area.type == 'OUTLINER'

    def get_selected_collection_name(self, context):
        for item in context.selected_ids:
            if item.bl_rna.identifier == "Collection":
                return item.name
                
    def execute(self, context):
        name = self.get_selected_collection_name(context)
        length = len(bpy.data.collections[name].all_objects)
        current_frame = bpy.context.scene.frame_current
        
        hide_viewport = bpy.data.collections[name].hide_viewport
        hide_render = bpy.data.collections[name].hide_render
        # print(f"{hide_viewport=},{hide_render=}")
        # task = "hide_viewport"
        # task = "hide_render"
        count = 0
        while count < length:
            bpy.data.collections[name].all_objects[count].hide_viewport = hide_viewport
            bpy.data.collections[name].all_objects[count].keyframe_insert(data_path="hide_viewport", frame=current_frame)
            
            bpy.data.collections[name].all_objects[count].hide_render = hide_render
            bpy.data.collections[name].all_objects[count].keyframe_insert(data_path="hide_render", frame=current_frame)
            
            count+=1
        return {'FINISHED'}
    
class Collection_Keyframe_Cleaner(bpy.types.Operator):
    bl_idname = "outliner.clear_renders_clear_viewpoints_created"
    bl_label = "clear_renders_clear_viewpoints_created"
    
    @classmethod
    def poll(cls, context):
        return context.area.type == 'OUTLINER'

    def get_selected_collection_name(self, context):
        for item in context.selected_ids:
            if item.bl_rna.identifier == "Collection":
                return item.name
                
    def execute(self, context):
        name = self.get_selected_collection_name(context)
        length = len(bpy.data.collections[name].all_objects)
        count = 0
        while count < length:
            try:
                remove_types = ["hide_viewport", "hide_render"]
                fcurves = [
                    fc for fc in bpy.data.collections[name].all_objects[count].animation_data.action.fcurves
                    for type in remove_types
                    if fc.data_path.startswith(type)
                ]
                # remove fcurves
                while(fcurves):
                    fc = fcurves.pop()
                    bpy.data.collections[name].all_objects[count].animation_data.action.fcurves.remove(fc)
            except:
                pass
            count+=1
        return {'FINISHED'}

#GUI
class KeyFrameAutomatorPanel(bpy.types.Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Animate Collection Visibility'
    bl_options = {'DEFAULT_CLOSED'}
    bl_idname = "outliner.key_frame_automator"
    bl_label = "Animate Collection Visibility"
    bl_space_type = "VIEW_3D"

    def draw(self, context):
        layout = self.layout  
        row = layout.row()      
        row.label(text="ANIMATE集合可见性的说明:")
        row = layout.row()      
        row.label(text="单击资产集合 (蓝色突出显示选择)")
        row = layout.row()
        row.label(text="该加载项将使集合中的所有内容具有相同的关键帧视口可见性和渲染可见性。")
        row = layout.row()
        row.label(text="Shift + Q 将关键帧添加到该集合内的所有集合和对象。")
        row = layout.row()
        row.label(text="Shift + T 将关键帧移除到该集合内的所有集合和对象。")
        row = layout.row()
        row.label(text="要打开视口可见性，请单击大纲视图右上角的过滤器图标.")
        row = layout.row()
        row.label(text="在视口或渲染模式下关闭的对象将不会计入GPU内存，并将使渲染速度更快。")
    
addon_keymaps = [] 
classes = (Collection_Iterator, KeyFrameAutomatorPanel, Collection_Keyframe_Cleaner)
def register():
    from bpy.utils import register_class
    # register_class(MessageBoxOperator)
    for cls in classes:
        register_class(cls)

    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = wm.keyconfigs.addon.keymaps.new(name='Window', space_type='EMPTY')
        kmi = km.keymap_items.new(Collection_Iterator.bl_idname, 'Q', 'PRESS', shift=True)
        addon_keymaps.append((km, kmi))
        km = wm.keyconfigs.addon.keymaps.new(name='Window', space_type='EMPTY')
        kmi = km.keymap_items.new(Collection_Keyframe_Cleaner.bl_idname, 'T', 'PRESS', shift=True)
        addon_keymaps.append((km, kmi))

if __name__ == "__main__":
    print("iinstalled")
    register()